@@ -111,19 +111,20 @@ class DistUpgradeQuirks(object):
         logging.debug("running Quirks.PreCacheOpen")
 
     # individual quirks handler that run *after* the cache is opened
-    def focalPostInitialUpdate(self):
+    def groovyPostInitialUpdate(self):
         # PreCacheOpen would be better but controller.abort fails terribly
         """ run after the apt cache is opened the first time """
-        logging.debug("running Quirks.focalPostInitialUpdate")
+        logging.debug("running Quirks.groovyPostInitialUpdate")
         self._get_from_and_to_version()
         self._test_and_fail_on_i386()
+
         cache = self.controller.cache
-        if 'ubuntu-desktop' not in cache or \
-                'snapd' not in cache:
+        self._test_and_warn_if_ros_installed(cache)
+
+        if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
             return
-        if cache['ubuntu-desktop'].is_installed and \
-                cache['snapd'].is_installed:
+        if cache['snapd'].is_installed:
             self._checkStoreConnectivity()
         # If the snap store is accessible, at the same time calculate the
         # extra size needed by to-be-installed snaps.  This also prepares
@@ -131,15 +132,13 @@ class DistUpgradeQuirks(object):
         if self._snapstore_reachable:
             self._calculateSnapSizeRequirements()
 
-    def focalPostUpgrade(self):
-        logging.debug("running Quirks.focalPostUpgrade")
+    def groovyPostUpgrade(self):
+        logging.debug("running Quirks.groovyPostUpgrade")
         cache = self.controller.cache
-        if 'ubuntu-desktop' not in cache or \
-                'snapd' not in cache:
+        if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
             return
-        if cache['ubuntu-desktop'].is_installed and \
-                cache['snapd'].is_installed and \
+        if cache['snapd'].is_installed and \
                 self._snap_list:
             self._replaceDebsAndSnaps()
 
@@ -363,6 +362,53 @@ class DistUpgradeQuirks(object):
                 self.controller.abort()
             self._view.processEvents()
 
+    def _test_and_warn_if_ros_installed(self, cache):
+        """
+        Test and warn if ROS is installed. A given ROS release only
+        supports specific Ubuntu releases, and can cause the upgrade
+        to fail in an overly-cryptic manner.
+        """
+
+        # These are the root ROS 1 and 2 dependencies as of 07/27/2020
+        ros_package_patterns = set()
+        for package_name in (
+                "catkin",
+                "rosboost-cfg",
+                "rosclean",
+                "ros-environment",
+                "ros-workspace"):
+            ros_package_patterns.add(
+                re.compile("ros-[^\-]+-%s" % package_name))
+
+        ros_is_installed = False
+        for pkg in cache:
+            if ros_is_installed:
+                break
+
+            for pattern in ros_package_patterns:
+                if pattern.match(pkg.name):
+                    if pkg.is_installed or pkg.marked_install:
+                        ros_is_installed = True
+                    break
+
+        if ros_is_installed:
+            res = self._view.askYesNoQuestion(
+                _("The Robot Operating System (ROS) is installed"),
+                _("It appears that ROS is currently installed. Each ROS "
+                  "release is very strict about the versions of Ubuntu "
+                  "it supports, and Ubuntu upgrades can fail if that "
+                  "guidance isn't followed. Before continuing, please "
+                  "either uninstall ROS, or ensure the ROS release you "
+                  "have installed supports the version of Ubuntu to "
+                  "which you're upgrading.\n\n"
+                  "For ROS 1 releases, refer to REP 3:\n"
+                  "https://www.ros.org/reps/rep-0003.html\n\n"
+                  "For ROS 2 releases, refer to REP 2000:\n"
+                  "https://www.ros.org/reps/rep-2000.html\n\n"
+                  "Are you sure you want to continue?"))
+            if not res:
+                self.controller.abort()
+
     def _checkArmCPU(self):
         """
         parse /proc/cpuinfo and search for ARMv6 or greater
@@ -848,22 +894,32 @@ class DistUpgradeQuirks(object):
         other package and the python-is-python2 package is installed instead,
         if python-minimal was installed.
         """
-        old = 'python-minimal'
-        new = 'python-is-python2'
+        # python-dbg must come first for reasons unknown
+        replacements = (('python-dbg', 'python2-dbg'),
+                        ('python-doc', 'python2-doc'),
+                        ('python-minimal', 'python-is-python2'),
+                        ('python-dev', 'python-dev-is-python2'),
+                        ('libpython-dev', None),
+                        ('libpython-stdlib', None),
+                        ('libpython-dbg', None))
         cache = self.controller.cache
-        if old in cache and cache[old].is_installed:
-            logging.info("installing %s because %s was installed" % (new, old))
-            reason = "%s was installed on the system" % old
-            if not cache.mark_install(new, reason):
-                logging.info("failed to install %s" % new)
-            logging.info("removing %s because %s is being installed" %
-                         (old, new))
-            reason = "%s is being installed on the system" % new
-            if not cache.mark_remove(old, reason):
-                logging.info("failed to remove %s", old)
+        for old, new in replacements:
+            logging.info("checking for %s" % old)
+            if old in cache and cache[old].is_installed:
+                if new:
+                    logging.info("installing %s because %s was installed" %
+                                 (new, old))
+                    reason = "%s was installed on the system" % old
+                    if not cache.mark_install(new, reason):
+                        logging.info("failed to install %s" % new)
+                logging.info("removing %s because %s is being installed" %
+                             (old, new))
+                reason = "%s is being installed on the system" % new
+                if not cache.mark_remove(old, reason):
+                    logging.info("failed to remove %s", old)
 
-            # protect our decision to remove legacy 'python' (as a dependency
-            # of python-minimal, removed above)
+            # protect our decision to remove legacy 'python' (as a
+            # dependency of python-minimal, removed above)
             py = 'python'
             if py in cache and cache[py].marked_delete:
                 resolver = apt.cache.ProblemResolver(cache)
@@ -900,6 +956,12 @@ class DistUpgradeQuirks(object):
 
             for snap in d2s["seeded"]:
                 seed = d2s["seeded"][snap]
+                metapkg = seed.get("metapkg", None)
+                if metapkg not in self.controller.cache:
+                    continue
+                if metapkg and \
+                        self.controller.cache[metapkg].is_installed is False:
+                    continue
                 deb = seed.get("deb", None)
                 from_chan = seed.get("from_channel", from_channel)
                 to_chan = seed.get("to_channel", to_channel)
@@ -908,11 +970,33 @@ class DistUpgradeQuirks(object):
             for snap in d2s["unseeded"]:
                 unseed = d2s["unseeded"][snap]
                 deb = unseed.get("deb", None)
+                metapkg = unseed.get("metapkg", None)
+                if metapkg not in self.controller.cache:
+                    continue
+                if metapkg and \
+                        self.controller.cache[metapkg].is_installed is False:
+                    continue
                 from_chan = unseed.get("from_channel", from_channel)
                 unseeded_snaps[snap] = (deb, from_chan)
         except Exception as e:
             logging.warning("error reading deb2snap.json file (%s)" % e)
 
+        snap_list = ''
+        # list the installed snaps and add them to seeded ones
+        snap_list = subprocess.Popen(["snap", "list"],
+                                     universal_newlines=True,
+                                     stdout=subprocess.PIPE).communicate()
+        if snap_list:
+            # first line of output is a header and the last line is empty
+            snaps_installed = [line.split()[0]
+                               for line in snap_list[0].split('\n')[1:-1]]
+
+            for snap in snaps_installed:
+                if snap in seeded_snaps or snap in unseeded_snaps:
+                    continue
+                else:
+                    seeded_snaps[snap] = (None, from_channel, to_channel)
+
         self._view.updateStatus(_("Checking for installed snaps"))
         for snap, (deb, from_channel, to_channel) in seeded_snaps.items():
             snap_object = {}
