@@ -30,12 +30,12 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
 
 from gettext import gettext as _
 import apt
-import fnmatch
 import logging
 import itertools
 import os
 import random
 import glob
+import re
 
 from gi.repository import Gio
 
@@ -210,7 +210,6 @@ class UpdateList():
     def __init__(self, parent, dist=None):
         self.dist = (dist if dist else utils.get_dist())
         self.distUpgradeWouldDelete = 0
-        self.oem_groups = []
         self.update_groups = []
         self.security_groups = []
         self.kernel_autoremove_groups = []
@@ -384,43 +383,6 @@ class UpdateList():
                 return True
         return False
 
-    def _get_linux_packages(self):
-        "Return all binary packages made by the linux-meta source package"
-        # Hard code this rather than generate from source info in cache because
-        # that might only be available if we have deb-src lines.  I think we
-        # could also generate it by iterating over all the binary package info
-        # we have, but that is costly.  These don't change often.
-        return ['linux',
-                'linux-cloud-tools-generic',
-                'linux-cloud-tools-lowlatency',
-                'linux-cloud-tools-virtual',
-                'linux-crashdump',
-                'linux-generic',
-                'linux-generic-lpae',
-                'linux-headers-generic',
-                'linux-headers-generic-lpae',
-                'linux-headers-lowlatency',
-                'linux-headers-lowlatency-lpae',
-                'linux-headers-server',
-                'linux-headers-virtual',
-                'linux-image',
-                'linux-image-extra-virtual',
-                'linux-image-generic',
-                'linux-image-generic-lpae',
-                'linux-image-lowlatency',
-                'linux-image-virtual',
-                'linux-lowlatency',
-                'linux-signed-generic',
-                'linux-signed-image-generic',
-                'linux-signed-image-lowlatency',
-                'linux-signed-lowlatency',
-                'linux-source',
-                'linux-tools-generic',
-                'linux-tools-generic-lpae',
-                'linux-tools-lowlatency',
-                'linux-tools-virtual',
-                'linux-virtual']
-
     def _make_groups(self, cache, pkgs, eventloop_callback, to_remove=False):
         if not pkgs:
             return []
@@ -452,15 +414,41 @@ class UpdateList():
         if ungrouped_pkgs:
             # Separate out system base packages. If we have already found an
             # application for all updates, don't bother.
-            meta_group = UpdateGroup(None, None, None, to_remove)
+            linux_names = ("linux$",
+                           "linux-.*-buildinfo.*",
+                           "linux-.*-dev.*",
+                           "linux-.*-generic.*",
+                           "linux-.*-headers.*",
+                           "linux-.*-hwe.*",
+                           "linux-.*-gcp.*",
+                           "linux-.*-kernel.*",
+                           "linux-.*-lowlatency.*",
+                           "linux-.*-modules.*",
+                           "linux-.*-raspi.*",
+                           "linux-.*-tools.*",
+                           "linux-.*-virtual.*",
+                           "linux-base.*",
+                           "linux-crashdump.*",
+                           "linux-doc.*")
+            linux_regexp = re.compile("(" + "|".join(
+                ["^" + n for n in linux_names]) + ")")
+            ubuntu_base_group = UpdateGroup(None, None, None, to_remove)
             flavor_package = utils.get_ubuntu_flavor_package(cache=cache)
-            meta_pkgs = [flavor_package, "ubuntu-standard", "ubuntu-minimal"]
-            meta_pkgs.extend(self._get_linux_packages())
-            for pkg in meta_pkgs:
-                if pkg in cache:
-                    meta_group.add(cache[pkg])
+            ubuntu_base_pkgs = [flavor_package,
+                                "ubuntu-standard",
+                                "ubuntu-minimal"]
+            for pkg in cache:
+                if linux_regexp.match(pkg.name):
+                    ubuntu_base_pkgs.append(pkg.name)
+            for pkg in ubuntu_base_pkgs:
+                if pkg in cache and not to_remove:
+                    # the check for to_remove avoids creating groups in
+                    # kernel_autoremove and duplicate package sections
+                    # which fixes LP: #1912718
+                    ubuntu_base_group.add(cache[pkg])
             for pkg in ungrouped_pkgs:
-                if meta_group.is_dependency(pkg, cache, eventloop_callback):
+                if ubuntu_base_group.is_dependency(pkg, cache,
+                                                   eventloop_callback):
                     if system_group is None:
                         system_group = UpdateSystemGroup(cache, to_remove)
                     system_group.add(pkg)
@@ -480,7 +468,6 @@ class UpdateList():
         # do the upgrade
         self.distUpgradeWouldDelete = cache.saveDistUpgrade()
 
-        oem_pkgs = []
         security_pkgs = []
         upgrade_pkgs = []
         kernel_autoremove_pkgs = []
@@ -505,9 +492,7 @@ class UpdateList():
                         self.ignored_phased_updates.append(pkg)
                         continue
 
-                if fnmatch.fnmatch(pkg.name, 'oem-*-meta'):
-                    oem_pkgs.append(pkg)
-                elif is_security_update:
+                if is_security_update:
                     security_pkgs.append(pkg)
                 else:
                     upgrade_pkgs.append(pkg)
@@ -541,8 +526,6 @@ class UpdateList():
                          security_pkgs + upgrade_pkgs
                          + kernel_autoremove_pkgs + duplicate_pkgs]
             self._populate_desktop_cache(pkg_names)
-        self.oem_groups = self._make_groups(cache, oem_pkgs,
-                                            eventloop_callback)
         self.update_groups = self._make_groups(cache, upgrade_pkgs,
                                                eventloop_callback)
         self.security_groups = self._make_groups(cache, security_pkgs,
