@@ -18,8 +18,7 @@ entry, the config entry will be skipped.
 
 **Module frequency:** per always
 
-**Supported distros:** almalinux, centos, eurolinux, fedora, photon, rhel,
-                       rocky, virtuozzo
+**Supported distros:** almalinux, centos, fedora, rhel
 
 **Config keys**::
 
@@ -37,8 +36,7 @@ from configparser import ConfigParser
 
 from cloudinit import util
 
-distros = ['almalinux', 'centos', 'eurolinux', 'fedora', 'photon', 'rhel',
-           'rocky', 'virtuozzo']
+distros = ['almalinux', 'centos', 'fedora', 'rhel']
 
 
 def _canonicalize_id(repo_id):
